# -*- coding: utf-8 -*-
""" Viewing components for WAeUP objects.
"""
import copy
import csv
import grok
import os
import re
import sys
import time
import re
import smtplib
from email.mime.text import MIMEText
from zope import schema
from zope.authentication.interfaces import (
    IAuthentication, IUnauthenticatedPrincipal, ILogout)
from zope.catalog.interfaces import ICatalog
from zope.securitypolicy.interfaces import (
    IPrincipalRoleManager, IPrincipalRoleMap)
from zope.component import (
    getUtility, createObject,getAllUtilitiesRegisteredFor)
#from zope.component.interfaces import Invalid
from zope.event import notify
from zope.session.interfaces import ISession
from waeup.sirp.browser import (
    WAeUPPage, WAeUPEditFormPage, WAeUPAddFormPage,
    WAeUPDisplayFormPage, NullValidator)
from waeup.sirp.browser.interfaces import (
    IUniversity, IFacultyContainer, IFaculty, IFacultyAdd,
    IDepartment, IDepartmentAdd, ICourse, ICourseAdd, ICertificate,
    ICertificateAdd, ICertificateCourse, ICertificateCourseAdd)
from waeup.sirp.browser.resources import warning, datepicker, tabs, datatable
from waeup.sirp.interfaces import(
    IWAeUPObject, IUsersContainer, IUserAccount, IDataCenter,
    IWAeUPXMLImporter, IWAeUPXMLExporter, IBatchProcessor,
    ILocalRolesAssignable, DuplicationError, IConfigurationContainer,
    ISessionConfiguration, ISessionConfigurationAdd, academic_sessions_vocab,
    IPasswordValidator)
from waeup.sirp.permissions import (
    get_users_with_local_roles,
    getGlobalRolesForAccount, getLocalRolesForAccount)
from waeup.sirp.university.catalog import search
from waeup.sirp.university.vocabularies import course_levels
from waeup.sirp.authentication import LocalRoleSetEvent
from waeup.sirp.widgets.restwidget import ReSTDisplayWidget
from waeup.sirp.authentication import get_principal_role_manager

grok.context(IWAeUPObject)
grok.templatedir('templates')

def add_local_role(view, tab, **data):
    localrole = view.request.form.get('local_role', None)
    user = view.request.form.get('user', None)
    if user is None or localrole is None:
        view.flash('No user selected.')
        view.redirect(view.url(view.context, '@@manage')+'#tab-%s' % tab)
        return
    role_manager = IPrincipalRoleManager(view.context)
    role_manager.assignRoleToPrincipal(localrole, user)
    notify(LocalRoleSetEvent(view.context, localrole, user, granted=True))
    view.redirect(view.url(view.context, u'@@manage')+'#tab-%s' % tab)
    return

def del_local_roles(view, tab, **data):
    child_ids = view.request.form.get('role_id', None)
    if child_ids is None:
        view.flash('No local role selected.')
        view.redirect(view.url(view.context, '@@manage')+'#tab-%s' % tab)
        return
    if not isinstance(child_ids, list):
        child_ids = [child_ids]
    deleted = []
    role_manager = IPrincipalRoleManager(view.context)
    for child_id in child_ids:
        localrole = child_id.split('|')[1]
        user_name = child_id.split('|')[0]
        try:
            role_manager.unsetRoleForPrincipal(localrole, user_name)
            notify(LocalRoleSetEvent(
                    view.context, localrole, user_name, granted=False))
            deleted.append(child_id)
        except:
            view.flash('Could not remove %s: %s: %s' % (
                    child_id, sys.exc_info()[0], sys.exc_info()[1]))
    if len(deleted):
        view.flash('Successfully removed: %s' % ', '.join(deleted))
    view.redirect(view.url(view.context, u'@@manage')+'#tab-%s' % tab)
    return

def delSubobjects(view, redirect, tab=None, subcontainer=None):
    form = view.request.form
    if form.has_key('val_id'):
        child_id = form['val_id']
    else:
        view.flash('No item selected.')
        if tab:
            view.redirect(view.url(view.context, redirect)+'#tab-%s' % tab)
        else:
            view.redirect(view.url(view.context, redirect))
        return
    if not isinstance(child_id, list):
        child_id = [child_id]
    deleted = []
    for id in child_id:
        try:
            if subcontainer:
                container = getattr(view.context, subcontainer, None)
                del container[id]
            else:
                del view.context[id]
            deleted.append(id)
        except:
            view.flash('Could not delete %s: %s: %s' % (
                    id, sys.exc_info()[0], sys.exc_info()[1]))
    if len(deleted):
        view.flash('Successfully removed: %s' % ', '.join(deleted))
    if tab:
        view.redirect(view.url(view.context, redirect)+'#tab-%s' % tab)
    else:
        view.redirect(view.url(view.context, redirect))
    return

#
# Login/logout pages...
#

class LoginPage(WAeUPPage):
    """A login page, available for all objects.
    """
    grok.name('login')
    grok.context(IWAeUPObject)
    grok.require('waeup.Public')
    title = u'Login'
    label = title
    camefrom = None

    def isStudent(self):
        prm = get_principal_role_manager()
        roles = [x[0] for x in prm.getRolesForPrincipal(self.request.principal.id)]
        return 'waeup.Student' in roles

    def update(self, SUBMIT=None, camefrom=None):
        self.camefrom = camefrom
        if SUBMIT is not None:
            if self.request.principal.id != 'zope.anybody':
                self.flash('You logged in.')
                if self.isStudent():
                    rel_link = '/students/%s' % self.request.principal.id
                    self.redirect(self.application_url() + rel_link)
                    return
                if not self.camefrom:
                    # User might have entered the URL directly. Let's beam
                    # him back to our context.
                    self.redirect(self.url(self.context))
                    return
                self.redirect(self.camefrom)
                return
            self.flash('You entered wrong credentials.')


class LogoutPage(WAeUPPage):
    """A logout page. Calling this page will log the current user out.
    """
    grok.context(IWAeUPObject)
    grok.require('waeup.Public')
    grok.name('logout')

    def update(self):
        if not IUnauthenticatedPrincipal.providedBy(self.request.principal):
            auth = getUtility(IAuthentication)
            ILogout(auth).logout(self.request)
            self.flash("You have been logged out. Thanks for using WAeUP SIRP!")
        self.redirect(self.application_url())

#
# University related pages...
#

class UniversityPage(WAeUPDisplayFormPage):
    """ The main university page.
    """
    grok.require('waeup.Public')
    grok.name('index')
    grok.context(IUniversity)
    pnav = 0

    @property
    def title(self):
        return "Welcome"

    @property
    def label(self):
        return self.context['configuration'].title

    @property
    def frontpage(self):
        view = ConfigurationContainerDisplayFormPage(
            self.context['configuration'],self.request)
        view.setUpWidgets()
        return view.widgets['frontpage']()

class AdministrationPage(WAeUPPage):
    """ The administration overview page.
    """
    grok.name('administration')
    grok.context(IUniversity)
    grok.require('waeup.manageUniversity')
    title = u'Administration'
    pnav = 0

class RSS20Feed(grok.View):
    """An RSS 2.0 feed.
    """
    grok.name('feed.rss')
    grok.context(IUniversity)
    grok.require('waeup.Public')
    grok.template('universityrss20feed')

    name = 'General news feed'
    description = 'waeup.sirp now supports RSS 2.0 feeds :-)'
    language = None
    date = None
    buildDate = None
    editor = None
    webmaster = None

    @property
    def title(self):
        return getattr(grok.getSite(), 'name', u'Sample University')

    @property
    def contexttitle(self):
        return self.name

    @property
    def link(self):
        return self.url(grok.getSite())

    def update(self):
        self.response.setHeader('Content-Type', 'text/xml; charset=UTF-8')

    def entries(self):
        return ()

#
# User container pages...
#

class UsersContainerPage(WAeUPPage):
    """Overview page for all local users.
    """
    grok.require('waeup.manageUsers')
    grok.context(IUsersContainer)
    grok.name('index')
    title = 'Portal Users'

    def update(self, userid=None, adduser=None, manage=None, delete=None):
        if manage is not None and userid is not None:
            self.redirect(self.url(userid) + '/@@manage')
        if delete is not None and userid is not None:
            self.context.delUser(userid)
            self.flash('User %s successfully deleted.' % userid)

    def getLocalRoles(self, account):
        return getLocalRolesForAccount(self, account)

    def getGlobalRoles(self, account):
        return getGlobalRolesForAccount(account)

class AddUserFormPage(WAeUPAddFormPage):
    grok.require('waeup.manageUsers')
    grok.context(IUsersContainer)
    grok.name('add')
    grok.template('usereditformpage')
    form_fields = grok.AutoFields(IUserAccount)
    label = 'Add user'
    title = 'Portal Users'

    @grok.action('Add user')
    def addUser(self, **data):
        name = data['name']
        title = data['title']
        description = data['description']
        #password = data['password']
        roles = data['roles']
        form = self.request.form
        password = form.get('password', None)
        password_ctl = form.get('control_password', None)
        if password:
            validator = getUtility(IPasswordValidator)
            errors = validator.validate_password(password, password_ctl)
            if errors:
                self.flash( ' '.join(errors))
                return
        try:
            self.context.addUser(name, password, title=title,
                                 description=description, roles=roles)
        except KeyError:
            self.status = self.flash('The userid chosen already exists '
                                  'in the database.')
            return
        self.redirect(self.url(self.context))

class UserManageFormPage(WAeUPEditFormPage):
    """Manage a user account.
    """
    grok.context(IUserAccount)
    grok.name('manage')
    grok.template('usereditformpage')
    grok.require('waeup.manageUsers')

    form_fields = grok.AutoFields(IUserAccount).omit('name')

    title = "Portal Users"

    def label(self):
        return "Edit user %s" % self.context.__name__

    @grok.action('Save')
    def save(self, **data):
        form = self.request.form
        password = form.get('password', None)
        password_ctl = form.get('control_password', None)
        if password:
            validator = getUtility(IPasswordValidator)
            errors = validator.validate_password(password, password_ctl)
            if errors:
                self.flash( ' '.join(errors))
                return
        self.applyData(self.context, **data)
        if password:
            # Now we know that the form has no errors and can set password ...
            self.context.setPassword(password)
        self.flash('User settings have been saved.')
        return

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self, **data):
        self.redirect(self.url(self.context.__parent__))
        return

class UserEditFormPage(UserManageFormPage):
    """Edit a user account by user
    """
    grok.name('index')
    grok.require('waeup.editUser')
    form_fields = grok.AutoFields(IUserAccount).omit('name', 'roles')

    def label(self):
        return "My preferences (username = %s)" % self.context.__name__

#
# Search pages...
#

class SearchPage(WAeUPPage):
    """General search page for the academics section.
    """
    grok.context(IFacultyContainer)
    grok.name('search')
    grok.template('searchpage')
    grok.require('waeup.manageUniversity')
    title = u"Academics"
    label = u"Search Academic Section"
    pnav = 1

    def update(self, *args, **kw):
        datatable.need()
        form = self.request.form
        self.hitlist = []
        self.query = ''
        if not 'query' in form:
            return
        query = form['query']
        self.query = query
        self.hitlist = search(query=self.query, view=self)
        return

#
# Contact forms...
#
class ContactAdminForm(WAeUPPage):
    grok.context(IUniversity)
    grok.name('contactadmin')
    grok.template('contactadminform')
    grok.require('waeup.Public')
    pnav = 2

    def title(self):
        """Return True if the calling user is authenticated.
        """
        userid = self.request.principal.id
        if userid != 'zope.anybody':
            tt = u'Contact'
        else:
            tt = u'Enquiries'
        return tt

    def label(self):
        return self.title()

    @property
    def namefield_label(self):
        if self.request.principal.title == 'Applicant':
            return 'Access Code'
        else:
            return 'Full Name'

    def update(self, *args, **kw):
        form = self.request.form
        if not ('fullname' in form and 'email' in form and 'descr' in form):
            return
        self.fullname = fullname = form['fullname']
        email = form['email']
        self.descr = descr = form['descr']
        regno =form['regno']
        if not (fullname and email and descr):
            self.flash('Error: All fields must be filled.')
            return
        if not re.match("^[a-zA-Z0-9._%-]+@[a-zA-Z0-9._%-]+.[a-zA-Z]{2,6}$",
                        email):
            self.flash('Error: %s is not a valid email address.' % email)
            return
        if regno == 'zope.anybody':
            regno = 'Anonymous User'
        text = """Fullname: %s
Member ID: %s
Description: %s
"""
        msg = MIMEText(text % (fullname,regno,descr))
        msg['From'] = '%s <%s>' % (fullname,email)

        # These parameters should be part of the portal configuration.
        msg['To'] = 'contact@waeup.org'
        msg['Subject'] = 'WAeUP Contact'
        #server = smtplib.SMTP('smtp_server')
        # Also connection details (username, passwords, host, port,
        # etc.) should be settable somewhere.
        # server.login('user_name','secret_password')
        server = smtplib.SMTP('localhost')
        server.sendmail(email,'contact@waeup.org',msg.as_string())

        server.quit()
        self.flash('Your message has been sent.')
        return


#
# Configuration pages...
#

class ConfigurationContainerDisplayFormPage(WAeUPDisplayFormPage):
    """View page of the configuration container.
    """
    grok.require('waeup.managePortalConfiguration')
    grok.name('view')
    grok.context(IConfigurationContainer)
    pnav = 0
    label = u'View portal configuration'
    title = u'Portal Configuration'
    form_fields = grok.AutoFields(IConfigurationContainer)
    form_fields['frontpage'].custom_widget = ReSTDisplayWidget

class ConfigurationContainerManageFormPage(WAeUPEditFormPage):
    """Manage page of the configuration container. We always use the
    manage page in the UI not the view page, thus we use the index name here.
    """
    grok.require('waeup.managePortalConfiguration')
    grok.name('index')
    grok.context(IConfigurationContainer)
    grok.template('configurationmanagepage')
    pnav = 0
    label = u'Edit portal configuration'
    title = u'Portal Configuration'
    taboneactions = ['Save', 'Update plugins']
    tabtwoactions = [
        'Add session configuration',
        'Remove selected']

    form_fields = grok.AutoFields(IConfigurationContainer)

    def update(self):
        tabs.need()
        datatable.need()
        return super(ConfigurationContainerManageFormPage, self).update()

    @grok.action('Save')
    def save(self, **data):
        self.applyData(self.context, **data)
        self.flash('Settings have been saved.')
        return

    @grok.action('Add session configuration', validator=NullValidator)
    def addSubunit(self, **data):
        self.redirect(self.url(self.context, '@@add'))
        return

    def getSessionConfigurations(self):
        """Get a list of all stored session configuration objects.
        """
        for key, val in self.context.items():
            url = self.url(val)
            session_string = val.getSessionString()
            title = 'Session %s Configuration' % session_string
            yield(dict(url=url, name=key, title=title))

    # ToDo: Show warning message before deletion
    @grok.action('Remove selected')
    def delSessonConfigurations(self, **data):
        delSubobjects(self, redirect='@@index', tab='2')
        return

    @grok.action('Update plugins', validator=NullValidator)
    def updatePlugins(self, **data):
        grok.getSite().updatePlugins()
        self.flash('Plugins were updated. See log file for details.')
        return

class SessionConfigurationAddFormPage(WAeUPAddFormPage):
    """Add a session configuration object to configuration container.
    """
    grok.context(IConfigurationContainer)
    grok.name('add')
    grok.require('waeup.managePortalConfiguration')
    label = 'Add session configuration'
    title = 'Configuration'
    form_fields = grok.AutoFields(ISessionConfigurationAdd)
    pnav = 0

    @grok.action('Add Session Configuration')
    def addSessionConfiguration(self, **data):
        sessionconfiguration = createObject(u'waeup.SessionConfiguration')
        self.applyData(sessionconfiguration, **data)
        try:
            self.context.addSessionConfiguration(sessionconfiguration)
        except KeyError:
            self.flash('The session chosen already exists ')
            return
        self.redirect(self.url(self.context, '@@index')+'#tab-2')
        return

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self):
        self.redirect(self.url(self.context, '@@index')+'#tab-2')
        return

class SessionConfigurationManageFormPage(WAeUPEditFormPage):
    """Manage session configuration object.
    """
    grok.context(ISessionConfiguration)
    grok.name('index')
    grok.require('waeup.managePortalConfiguration')
    form_fields = grok.AutoFields(ISessionConfiguration)
    pnav = 0

    @property
    def title(self):
        session_string = self.context.getSessionString()
        return 'Academic Session %s Configuration' % session_string

    @property
    def label(self):
        session_string = self.context.getSessionString()
        return 'Edit academic session %s configuration' % session_string

    @grok.action('Save')
    def save(self, **data):
        self.applyData(self.context, **data)
        self.flash('Settings have been saved.')
        self.redirect(self.url(self.context.__parent__, '@@index')+'#tab-2')
        return

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self):
        self.redirect(self.url(self.context.__parent__, '@@index')+'#tab-2')
        return

#
# Datacenter pages...
#

class DatacenterPage(WAeUPPage):
    grok.context(IDataCenter)
    grok.name('index')
    grok.require('waeup.manageUniversity')
    title = u'Data Center'
    pnav = 0

class DatacenterUploadPage(WAeUPPage):
    grok.context(IDataCenter)
    grok.name('upload')
    grok.require('waeup.manageUniversity')
    title = u'Data Center'
    label = u'Upload file'
    pnav = 0

    def update(self, uploadfile=None, CANCEL=None, SUBMIT=None):
        if CANCEL is not None:
            self.redirect(self.url(self.context))
            return
        if not uploadfile:
            return
        try:
            filename = uploadfile.filename
            target = os.path.join(self.context.storage,
                                  self.getNormalizedFileName(filename))
            open(target, 'wb').write(uploadfile.read())
            logger = self.context.logger
            logger.info('%s: Uploaded file "%s"' % (
                self.request.principal.id, target))
        except IOError:
            self.flash('Error while uploading file. Please retry.')
            self.flash('I/O error: %s' % sys.exc_info()[1])
            return
        self.redirect(self.url(self.context))

    def getNormalizedFileName(self, filename):
        """Build sane filename.

        An uploaded file foo.csv will be stored as foo_USERNAME.csv
        where username is the principal id of the currently logged in
        user.

        Spaces in filename are replaced by underscore.
        """
        username = self.request.principal.id
        filename = filename.replace(' ', '_')
        # Only accept typical filname chars...
        filtered_username = ''.join(re.findall('[a-zA-Z0-9_\.\-]', username))
        base, ext = os.path.splitext(filename)
        return '%s_%s%s' % (base, filtered_username, ext.lower())

class DatacenterImportStep1(WAeUPPage):
    """Manual import step 1: choose file
    """
    grok.context(IDataCenter)
    grok.name('import1')
    grok.template('datacenterimport1page')
    grok.require('waeup.manageUniversity')
    title = u'Data Center'
    label = u'Process CSV file'
    pnav = 0

    def getFiles(self):
        files = self.context.getFiles(sort='date')
        for file in files:
            name = file.name
            if not name.endswith('.csv') and not name.endswith('.pending'):
                continue
            yield file

    def update(self, filename=None, select=None, cancel=None):
        if cancel is not None:
            self.flash('Import aborted.')
            self.redirect(self.url(self.context))
            return
        if select is not None:
            # A filename was selected
            session = ISession(self.request)['waeup.sirp']
            session['import_filename'] = select
            self.redirect(self.url(self.context, '@@import2'))

class DatacenterImportStep2(WAeUPPage):
    """Manual import step 2: choose importer
    """
    grok.context(IDataCenter)
    grok.name('import2')
    grok.template('datacenterimport2page')
    grok.require('waeup.manageUniversity')
    title = u'Data Center'
    label = u'Process CSV file'
    pnav = 0

    filename = None
    mode = 'create'
    importer = None
    mode_locked = False

    def getPreviewHeader(self):
        """Get the header fields of attached CSV file.
        """
        reader = csv.reader(open(self.fullpath, 'rb'))
        return reader.next()

    def getPreviewBody(self):
        """Get the first 5 rows of attached CSV file.
        """
        result = []
        num = 0
        if not self.reader:
            return
        for row in self.reader:
            if num > 4:
                break
            num += 1
            row = row.items()
            # Sort fields in headerfield order
            row = sorted(row,
                         key=lambda k: self.reader.fieldnames.index(k[0]))
            row = [x[1] for x in row]
            result.append(row)
        result.append(len(result[0]) * ['...'])
        return result

    def getImporters(self):
        importers = getAllUtilitiesRegisteredFor(IBatchProcessor)
        importers = [
            dict(title=x.name, name=x.util_name) for x in importers]
        return importers

    def getModeFromFilename(self, filename):
        """Lookup filename or path and return included mode name or None.
        """
        if not filename.endswith('.pending.csv'):
            return None
        base = os.path.basename(filename)
        parts = base.rsplit('.', 3)
        if len(parts) != 4:
            return None
        if parts[1] not in ['create', 'update', 'remove']:
            return None
        return parts[1]

    def getWarnings(self):
        import sys
        result = []
        try:
            headerfields = self.getPreviewHeader()
            headerfields_clean = list(set(headerfields))
            if len(headerfields) > len(headerfields_clean):
                result.append(
                    "Double headers: each column name may only appear once. ")
        except:
            fatal = '%s' % sys.exc_info()[1]
            result.append(fatal)
        if result:
            warnings = ""
            for line in result:
                warnings += line + '<br />'
            warnings += 'Replace imported file!'
            return warnings
        return False

    def update(self, mode=None, importer=None,
               back1=None, cancel=None, proceed=None):
        session = ISession(self.request)['waeup.sirp']
        self.filename = session.get('import_filename', None)

        if self.filename is None or back1 is not None:
            self.redirect(self.url(self.context, '@@import1'))
            return
        if cancel is not None:
            self.flash('Import aborted.')
            self.redirect(self.url(self.context))
            return
        self.mode = mode or session.get('import_mode', self.mode)
        filename_mode = self.getModeFromFilename(self.filename)
        if filename_mode is not None:
            self.mode = filename_mode
            self.mode_locked = True
        self.importer = importer or session.get('import_importer', None)
        session['import_importer'] = self.importer
        if self.importer and 'update' in self.importer:
            if self.mode != 'update':
                self.flash('Update mode only!')
                self.mode_locked = True
                self.mode = 'update'
                proceed = None
        session['import_mode'] = self.mode
        if proceed is not None:
            self.redirect(self.url(self.context, '@@import3'))
            return
        self.fullpath = os.path.join(self.context.storage, self.filename)
        warnings = self.getWarnings()
        if not warnings:
            self.reader = csv.DictReader(open(self.fullpath, 'rb'))
        else:
            self.reader = ()
            self.flash(warnings)

class DatacenterImportStep3(WAeUPPage):
    """Manual import step 3: modify header
    """
    grok.context(IDataCenter)
    grok.name('import3')
    grok.template('datacenterimport3page')
    grok.require('waeup.manageUniversity')
    title = u'Data Center'
    label = u'Process CSV file'
    pnav = 0

    filename = None
    mode = None
    importername = None

    @property
    def nextstep(self):
        return self.url(self.context, '@@import4')

    def getPreviewHeader(self):
        """Get the header fields of attached CSV file.
        """
        reader = csv.reader(open(self.fullpath, 'rb'))
        return reader.next()

    def getPreviewBody(self):
        """Get the first 5 rows of attached CSV file.
        """
        result = []
        num = 0
        for row in self.reader:
            if num > 4:
                break
            num += 1
            row = row.items()
            # Sort fields in headerfield order
            row = sorted(row,
                         key=lambda k: self.reader.fieldnames.index(k[0]))
            row = [x[1] for x in row]
            result.append(row)
        result.append(len(result[0]) * ['...'])
        return result

    def getPossibleHeaders(self):
        """Get the possible headers.

        The headers are described as dicts {value:internal_name,
        title:displayed_name}
        """
        result = [dict(title='<IGNORE COL>', value='--IGNORE--')]
        headers = self.importer.getHeaders()
        result.extend([dict(title=x, value=x) for x in headers])
        return result

    def getWarnings(self):
        import sys
        result = []
        try:
            self.importer.checkHeaders(self.headerfields, mode=self.mode)
        except:
            fatal = '%s' % sys.exc_info()[1]
            result.append(fatal)
        if result:
            warnings = ""
            for line in result:
                warnings += line + '<br />'
            warnings += 'Edit headers or replace imported file!'
            return warnings
        return False

    @property
    def nextstep(self):
        return self.url(self.context, '@@import4')

    def update(self, headerfield=None, back2=None, cancel=None, proceed=None):
        session = ISession(self.request)['waeup.sirp']
        self.filename = session.get('import_filename', None)
        self.mode = session.get('import_mode', None)
        self.importername = session.get('import_importer', None)

        if None in (self.filename, self.mode, self.importername):
            self.redirect(self.url(self.context, '@@import2'))
            return
        if back2 is not None:
            self.redirect(self.url(self.context ,'@@import2'))
            return
        if cancel is not None:
            self.flash('Import aborted.')
            self.redirect(self.url(self.context))
            return

        self.fullpath = os.path.join(self.context.storage, self.filename)
        self.headerfields = headerfield or self.getPreviewHeader()
        session['import_headerfields'] = self.headerfields

        if proceed is not None:
            self.redirect(self.url(self.context, '@@import4'))
            return
        self.importer = getUtility(IBatchProcessor, name=self.importername)
        self.reader = csv.DictReader(open(self.fullpath, 'rb'))
        warnings = self.getWarnings()
        if warnings:
            self.flash(warnings)

class DatacenterImportStep4(WAeUPPage):
    """Manual import step 4: do actual import
    """
    grok.context(IDataCenter)
    grok.name('import4')
    grok.template('datacenterimport4page')
    grok.require('waeup.manageUniversity')
    title = u'Data Center'
    label = u'Process CSV file'
    pnav = 0

    filename = None
    mode = None
    importername = None
    headerfields = None
    warnnum = None

    def update(self, back=None, finish=None, showlog=None):
        if finish is not None:
            self.redirect(self.url(self.context))
            return
        session = ISession(self.request)['waeup.sirp']
        self.filename = session.get('import_filename', None)
        self.mode = session.get('import_mode', None)
        self.importername = session.get('import_importer', None)
        self.headerfields = session.get('import_headerfields', None)

        if None in (self.filename, self.mode, self.importername,
                    self.headerfields):
            self.redirect(self.url(self.context, '@@import3'))
            return

        if showlog is not None:
            logfilename = "datacenter.log"
            session['logname'] = logfilename
            self.redirect(self.url(self.context, '@@show'))
            return

        self.fullpath = os.path.join(self.context.storage, self.filename)
        self.importer = getUtility(IBatchProcessor, name=self.importername)

        # Perform batch processing...
        # XXX: This might be better placed in datacenter module.
        (linenum, self.warn_num,
         fin_path, pending_path) = self.importer.doImport(
            self.fullpath, self.headerfields, self.mode,
            self.request.principal.id, logger=self.context.logger)
        # Put result files in desired locations...
        self.context.distProcessedFiles(
            self.warn_num == 0, self.fullpath, fin_path, pending_path,
            self.mode)

        if self.warn_num:
            self.flash('Processing of %d rows failed.' % self.warn_num)
        self.flash('Successfully processed %s rows.' % (
                linenum - (self.warn_num)))

class DatacenterLogsOverview(WAeUPPage):
    grok.context(IDataCenter)
    grok.name('logs')
    grok.template('datacenterlogspage')
    grok.require('waeup.manageUniversity')
    title = u'Data Center'
    label = u'Show logfiles'
    pnav = 0

    def update(self, show=None, logname=None, back=None):
        session = ISession(self.request)['waeup.sirp']
        if back is not None:
            self.redirect(self.url(self.context))
            return
        if logname is not None:
            session['logname'] = logname
        if show is not None:
            self.redirect(self.url(self.context, '@@show'))
            return
        self.files = self.context.getLogFiles()

class DatacenterLogsFileview(WAeUPPage):
    grok.context(IDataCenter)
    grok.name('show')
    grok.template('datacenterlogsshowfilepage')
    grok.require('waeup.manageUniversity')
    title = u'Data Center'
    pnav = 0

    def update(self, show=None, remove=None, back=None):
        session = ISession(self.request)['waeup.sirp']
        logname = session.get('logname', None)
        if back is not None or logname is None:
            self.redirect(self.url(self.context, '@@logs'))
            return
        self.filename = logname
        self.files = self.context.getLogFiles()
        fullpath = os.path.join(self.context.storage, 'logs', logname)
        self.filecontents = open(fullpath, 'rb').read()

class DatacenterSettings(WAeUPPage):
    grok.context(IDataCenter)
    grok.name('manage')
    grok.template('datacentermanagepage')
    grok.require('waeup.manageUniversity')
    title = u'Data Center'
    label = 'Edit data center settings'
    pnav = 0

    def update(self, newpath=None, move=False, overwrite=False,
               save=None, cancel=None):
        if move:
            move = True
        if overwrite:
            overwrite = True
        if newpath is None:
            return
        if cancel is not None:
            self.redirect(self.url(self.context))
            return
        try:
            not_copied = self.context.setStoragePath(newpath, move=move)
            for name in not_copied:
                self.flash('File already existed (not copied): %s' % name)
        except:
            self.flash('Given storage path cannot be used.')
            self.flash('Error: %s' %sys.exc_info()[1])
            return
        if newpath:
            self.flash('New storage path succefully set.')
            self.redirect(self.url(self.context))
        return

class ExportXMLPage(grok.View):
    """Deliver an XML representation of the context.
    """
    grok.name('export.xml')
    grok.require('waeup.manageUniversity')

    def render(self):
        exporter = IWAeUPXMLExporter(self.context)
        xml = exporter.export().read()
        self.response.setHeader(
            'Content-Type', 'text/xml; charset=UTF-8')
        return xml

class ImportXMLPage(WAeUPPage):
    """Replace the context object by an object created from an XML
       representation.

       XXX: This does not work for ISite objects, i.e. for instance
            for complete University objects.
    """
    grok.name('importxml')
    grok.require('waeup.manageUniversity')

    def update(self, xmlfile=None, CANCEL=None, SUBMIT=None):
        if CANCEL is not None:
            self.redirect(self.url(self.context))
            return
        if not xmlfile:
            return
        importer = IWAeUPXMLImporter(self.context)
        obj = importer.doImport(xmlfile)
        if type(obj) != type(self.context):
            return
        parent = self.context.__parent__
        name = self.context.__name__
        self.context = obj
        if hasattr(parent, name):
            setattr(parent, name, obj)
        else:
            del parent[name]
            parent[name] = obj
            pass
        return



#
# FacultyContainer pages...
#

class FacultyContainerPage(WAeUPPage):
    """ Index page for faculty containers.
    """
    grok.context(IFacultyContainer)
    grok.require('waeup.View')
    grok.name('index')
    title = 'Academics'
    label = 'Academic Section'
    pnav = 1
    grok.template('facultypage')

class FacultyContainerManageFormPage(WAeUPEditFormPage):
    """Manage the basic properties of a `Faculty` instance.
    """
    grok.context(IFacultyContainer)
    grok.name('manage')
    grok.require('waeup.manageUniversity')
    grok.template('facultycontainermanagepage')
    pnav = 1
    taboneactions = ['Add faculty', 'Remove selected','Cancel']
    subunits = 'Faculties'
    title = 'Academics'

    @property
    def label(self):
        return 'Manage academic section'

    def update(self):
        tabs.need()
        #warning.need()
        return super(FacultyContainerManageFormPage, self).update()

    # ToDo: Show warning message before deletion
    @grok.action('Remove selected')
    def delFaculties(self, **data):
        delSubobjects(self, redirect='@@manage', tab='1')
        return

    @grok.action('Add faculty', validator=NullValidator)
    def addFaculty(self, **data):
        self.redirect(self.url(self.context, '@@add'))
        return

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return


class FacultyAddFormPage(WAeUPAddFormPage):
    """ Page form to add a new faculty to a faculty container.
    """
    grok.context(IFacultyContainer)
    grok.require('waeup.manageUniversity')
    grok.name('add')
    label = 'Add faculty'
    title = 'Academics'
    form_fields = grok.AutoFields(IFacultyAdd)
    pnav = 1

    @grok.action('Add faculty')
    def addFaculty(self, **data):
        faculty = createObject(u'waeup.Faculty')
        self.applyData(faculty, **data)
        try:
            self.context.addFaculty(faculty)
        except KeyError:
            self.flash('The faculty code chosen already exists.')
            return
        self.redirect(self.url(self.context, u'@@manage')+'#tab-1')

    @grok.action('Cancel')
    def cancel(self, **data):
        self.redirect(self.url(self.context))

#
# Faculty pages
#
class FacultyPage(WAeUPPage):
    """Index page of faculties.
    """
    grok.context(IFaculty)
    grok.require('waeup.View')
    grok.name('index')
    pnav = 1

    @property
    def title(self):
        return self.context.longtitle()

    @property
    def label(self):
        return 'Departments'

class FacultyManageFormPage(WAeUPEditFormPage):
    """Manage the basic properties of a `Faculty` instance.
    """
    grok.context(IFaculty)
    grok.name('manage')
    grok.require('waeup.manageUniversity')
    grok.template('facultymanagepage')
    pnav = 1
    subunits = 'Departments'
    taboneactions = ['Save','Cancel']
    tabtwoactions = ['Add department', 'Remove selected','Cancel']
    tabthreeactions1 = ['Remove selected local roles']
    tabthreeactions2 = ['Add local role']

    form_fields = grok.AutoFields(IFaculty)

    @property
    def label(self):
        return 'Manage faculty'

    @property
    def title(self):
        return self.context.longtitle()

    def update(self):
        tabs.need()
        datatable.need()
        return super(FacultyManageFormPage, self).update()

    def getLocalRoles(self):
        roles = ILocalRolesAssignable(self.context)
        return roles()

    def getUsers(self):
        """Get a list of all users.
        """
        for key, val in grok.getSite()['users'].items():
            url = self.url(val)
            yield(dict(url=url, name=key, val=val))

    def getUsersWithLocalRoles(self):
        return get_users_with_local_roles(self.context)

    # ToDo: Show warning message before deletion
    @grok.action('Remove selected')
    def delDepartments(self, **data):
        delSubobjects(self, redirect='@@manage', tab='2')
        return

    @grok.action('Save')
    def save(self, **data):
        self.applyData(self.context, **data)
        return

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return

    @grok.action('Add department', validator=NullValidator)
    def addSubunit(self, **data):
        self.redirect(self.url(self.context, '@@add'))
        return

    @grok.action('Add local role', validator=NullValidator)
    def addLocalRole(self, **data):
        return add_local_role(self, '3', **data)

    @grok.action('Remove selected local roles')
    def delLocalRoles(self, **data):
        return del_local_roles(self,3,**data)

class DepartmentAddFormPage(WAeUPAddFormPage):
    """Add a department to a faculty.
    """
    grok.context(IFaculty)
    grok.name('add')
    grok.require('waeup.manageUniversity')
    label = 'Add department'
    form_fields = grok.AutoFields(IDepartmentAdd)
    pnav = 1

    @property
    def title(self):
        return self.context.longtitle()

    @grok.action('Add department')
    def addDepartment(self, **data):
        department = createObject(u'waeup.Department')
        self.applyData(department, **data)
        try:
            self.context.addDepartment(department)
        except KeyError:
            self.flash('The code chosen already exists '
                                  'in this faculty.')
            return
        self.redirect(self.url(self.context, u'@@manage')+'#tab-2')

    @grok.action('Cancel')
    def cancel(self, **data):
        self.redirect(self.url(self.context))

#
# Department pages
#
class DepartmentPage(WAeUPPage):
    """Department index page.
    """
    grok.context(IDepartment)
    grok.require('waeup.View')
    grok.name('index')
    pnav = 1
    label = 'Courses and Certificates'

    @property
    def title(self):
        return self.context.longtitle()

    def update(self):
        tabs.need()
        datatable.need()
        super(DepartmentPage, self).update()
        return

    def getCourses(self):
        """Get a list of all stored courses.
        """
        for key, val in self.context.courses.items():
            url = self.url(val)
            yield(dict(url=url, name=key, container=val))

    def getCertificates(self):
        """Get a list of all stored certificates.
        """
        for key, val in self.context.certificates.items():
            url = self.url(val)
            yield(dict(url=url, name=key, container=val))

class DepartmentManageFormPage(WAeUPEditFormPage):
    """Manage the basic properties of a `Department` instance.
    """
    grok.context(IDepartment)
    grok.name('manage')
    grok.require('waeup.manageUniversity')
    pnav = 1
    grok.template('departmentmanagepage')
    taboneactions = ['Save','Cancel']
    tabtwoactions = ['Add course', 'Remove selected courses','Cancel']
    tabthreeactions = ['Add certificate', 'Remove selected certificates',
                       'Cancel']
    tabfouractions1 = ['Remove selected local roles']
    tabfouractions2 = ['Add local role']

    form_fields = grok.AutoFields(IDepartment)

    @property
    def label(self):
        return 'Manage department'

    @property
    def title(self):
        return self.context.longtitle()

    def getCourses(self):
        """Get a list of all stored courses.
        """
        for key, val in self.context.courses.items():
            url = self.url(val)
            yield(dict(url=url, name=key, container=val))

    def getCertificates(self):
        """Get a list of all stored certificates.
        """
        for key, val in self.context.certificates.items():
            url = self.url(val)
            yield(dict(url=url, name=key, container=val))

    def update(self):
        tabs.need()
        datatable.need()
        super(DepartmentManageFormPage, self).update()
        return

    def getLocalRoles(self):
        roles = ILocalRolesAssignable(self.context)
        return roles()

    def getUsers(self):
        """Get a list of all users.
        """
        for key, val in grok.getSite()['users'].items():
            url = self.url(val)
            yield(dict(url=url, name=key, val=val))

    def getUsersWithLocalRoles(self):
        return get_users_with_local_roles(self.context)

    @grok.action('Save')
    def save(self, **data):
        self.applyData(self.context, **data)
        return

    # ToDo: Show warning message before deletion
    @grok.action('Remove selected courses')
    def delCourses(self, **data):
        delSubobjects(
            self, redirect='@@manage', tab='2', subcontainer='courses')
        return

    @grok.action('Remove selected certificates')
    def delCertificates(self, **data):
        delSubobjects(
            self, redirect='@@manage', tab='3', subcontainer='certificates')
        return

    @grok.action('Add course', validator=NullValidator)
    def addCourse(self, **data):
        self.redirect(self.url(self.context, 'addcourse'))
        return

    @grok.action('Add certificate', validator=NullValidator)
    def addCertificate(self, **data):
        self.redirect(self.url(self.context, 'addcertificate'))
        return

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return

    @grok.action('Add local role', validator=NullValidator)
    def addLocalRole(self, **data):
        return add_local_role(self, 4, **data)

    @grok.action('Remove selected local roles')
    def delLocalRoles(self, **data):
        return del_local_roles(self,4,**data)

class CourseAddFormPage(WAeUPAddFormPage):
    """Add-form to add course to a department.
    """
    grok.context(IDepartment)
    grok.name('addcourse')
    grok.require('waeup.manageUniversity')
    label = u'Add course'
    form_fields = grok.AutoFields(ICourseAdd)
    pnav = 1

    @property
    def title(self):
        return self.context.longtitle()

    @grok.action('Add course')
    def addCourse(self, **data):
        course = createObject(u'waeup.Course')
        self.applyData(course, **data)
        try:
            self.context.courses.addCourse(course)
        except DuplicationError, error:
            # signals duplication error
            entries = error.entries
            for entry in entries:
                # do not use error.msg but render a more detailed message instead
                # and show links to all certs with same code
                message = 'A course with same code already exists: '
                message += '<a href="%s">%s</a>' % (
                    self.url(entry), getattr(entry, '__name__', u'Unnamed'))
                self.flash(message)
            self.redirect(self.url(self.context, u'@@addcourse'))
            return
        message = u'Course %s successfully created.' % (course.code)
        self.flash(message)
        self.redirect(self.url(self.context, u'@@manage')+'#tab-2')

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return

class CertificateAddFormPage(WAeUPAddFormPage):
    """Add-form to add certificate to a department.
    """
    grok.context(IDepartment)
    grok.name('addcertificate')
    grok.require('waeup.manageUniversity')
    label = u'Add certificate'
    form_fields = grok.AutoFields(ICertificateAdd)
    pnav = 1

    @property
    def title(self):
        return self.context.longtitle()

    @grok.action('Add certificate')
    def addCertificate(self, **data):
        certificate = createObject(u'waeup.Certificate')
        self.applyData(certificate, **data)
        try:
            self.context.certificates.addCertificate(certificate)
        except DuplicationError, error:
            # signals duplication error
            entries = error.entries
            for entry in entries:
                # do not use error.msg but render a more detailed message instead
                # and show links to all certs with same code
                message = 'A certificate with same code already exists: '
                message += '<a href="%s">%s</a>' % (
                    self.url(entry), getattr(entry, '__name__', u'Unnamed'))
                self.flash(message)
            self.redirect(self.url(self.context, u'@@addcertificate'))
            return
        message = u'Certificate %s successfully created.' % (certificate.code)
        self.flash(message)
        self.redirect(self.url(self.context, u'@@manage')+'#tab-3')
        return

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self): #, **data):
        self.redirect(self.url(self.context))
        return

#
# Courses pages
#
class CoursePage(WAeUPPage):
    """Course index page.
    """
    grok.context(ICourse)
    grok.name('index')
    grok.require('waeup.View')
    pnav = 1
    #form_fields = grok.AutoFields(ICourse)

    @property
    def title(self):
        return 'Course: %s (%s)' % (self.context.title, self.context.code)

class CourseManageFormPage(WAeUPEditFormPage):
    """Edit form page for courses.
    """
    grok.context(ICourse)
    grok.name('manage')
    grok.require('waeup.manageUniversity')
    label = u'Edit course'
    pnav = 1

    form_fields = grok.AutoFields(ICourse)

    @property
    def title(self):
        return 'Course: %s (%s)' % (self.context.title, self.context.code)

    @grok.action('Save')
    def save(self, **data):
        self.applyData(self.context, **data)
        return

    @grok.action('Save and return')
    def saveAndReturn(self, **data):
        self.applyData(self.context, **data)
        self.redirect(self.url(self.context))
        return

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return

#
# Certificate pages
#
class CertificatePage(WAeUPDisplayFormPage):
    """Index page for certificates.
    """
    grok.context(ICertificate)
    grok.name('index')
    grok.require('waeup.View')
    pnav = 1
    form_fields = grok.AutoFields(ICertificate)
    grok.template('certificatepage')

    @property
    def title(self):
        return "Certificate: %s (%s)" % (self.context.title, self.context.code)

    def update(self):
        tabs.need()
        #warning.need()
        datatable.need()
        return super(CertificatePage, self).update()

class CertificateManageFormPage(WAeUPEditFormPage):
    """Manage the properties of a `Certificate` instance.
    """
    grok.context(ICertificate)
    grok.name('manage')
    grok.require('waeup.manageUniversity')
    pnav = 1
    label = 'Edit certificate'

    form_fields = grok.AutoFields(ICertificate)

    pnav = 1
    grok.template('certificatemanagepage')
    taboneactions = ['Save','Cancel']
    tabtwoactions = ['Add course referrer',
                     'Remove selected course referrers','Cancel']

    @property
    def label(self):
        return 'Manage certificate'

    @property
    def title(self):
        return "Certificate: %s (%s)" % (self.context.title, self.context.code)

    def update(self):
        tabs.need()
        #warning.need()
        datatable.need()
        return super(CertificateManageFormPage, self).update()

    form_fields = grok.AutoFields(ICertificate)

    @grok.action('Save')
    def save(self, **data):
        self.applyData(self.context, **data)
        return

    # ToDo: Show warning message before deletion
    @grok.action('Remove selected course referrers')
    def delCertificateCourses(self, **data):
        delSubobjects(self, redirect='@@manage', tab='2')
        return

    @grok.action('Add course referrer', validator=NullValidator)
    def addCertificateCourse(self, **data):
        self.redirect(self.url(self.context, 'addcertificatecourse'))
        return

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return


class CertificateCourseAddFormPage(WAeUPAddFormPage):
    """Add-page to add a course ref to a certificate
    """
    grok.context(ICertificate)
    grok.name('addcertificatecourse')
    grok.require('waeup.manageUniversity')
    form_fields = grok.AutoFields(ICertificateCourseAdd)
    pnav = 1
    label = 'Add course referrer'

    @property
    def title(self):
        return "Certificate: %s (%s)" % (self.context.title, self.context.code)

    @grok.action('Add course referrer')
    def addCertcourse(self, **data):
        try:
            self.context.addCourseRef(**data)
        except KeyError:
            self.status = self.flash('The chosen course referrer is already '
                                  'part of this certificate.')
            return
        self.redirect(self.url(self.context, u'@@manage')+'#tab-2')

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return


#
# Certificate course pages...
#
class CertificateCoursePage(WAeUPPage):
    """CertificateCourse index page.
    """
    grok.context(ICertificateCourse)
    grok.name('index')
    grok.require('waeup.View')
    pnav = 1
    #form_fields = grok.AutoFields(ICertificateCourse)

    @property
    def title(self):
        return 'Course Referrer: %s' % (self.context.longtitle())

    @property
    def leveltitle(self):
        return course_levels.getTerm(self.context.level).title

class CertificateCourseManageFormPage(WAeUPEditFormPage):
    """Manage the basic properties of a `CertificateCourse` instance.
    """
    grok.context(ICertificateCourse)
    grok.name('manage')
    grok.require('waeup.manageUniversity')
    form_fields = grok.AutoFields(ICertificateCourse)
    label = 'Edit course referrer'
    pnav = 1

    @property
    def title(self):
        return 'Course Referrer: %s' % (self.context.__name__)

    @grok.action('Save and return')
    def saveAndReturn(self, **data):
        parent = self.context.__parent__
        if self.context.level == data['level']:
            self.applyData(self.context, **data)
        else:
            # The level changed. We have to create a new certcourse as
            # the key of the entry will change as well...
            old_level = self.context.level
            data['course'] = self.context.course
            parent.addCourseRef(**data)
            parent.delCourseRef(data['course'].code, level=old_level)
        self.redirect(self.url(parent))
        return

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return

